'*******************************************************************************
'***  Title: Main Computer Board Code                                    ***
'***  Filename: Polaroid4.bas                                            ***
'***  Author: Colin O'Flynn                                              ***
'***  Date of creation: March 13, 200                                    ***
'***  Date of last revision:                                             ***
'***************************************************************************
'***  Created for: AT90S8535                                             ***
'***************************************************************************
'***                         What this does                              ***
'*** This is the "Main Computer Board", also called the "Polaroid board".***
'*** There no actual reason for calling it this, just "happened". This   ***
'*** board does several functions. First it controls an ultrasonic       ***
'*** rangefinder and uses it to measure distance. It communicates with   ***
'*** the ANN board and the speed control, a sort of communications hub.  ***
'*** It also controls the rudder.                                        ***
'***************************************************************************
'*** This code contains the following blocks of the Control Block Diagram***
'*** Transducer controller, distance calulation, Servo 2 controller      ***
'*** Turn direction logic, Rudder position calculator, Servo 1 controller***
'*** PWM calculator 1                                                    ***
'***************************************************************************



'Configure Device
$crystal = 4000000
$baud = 9600

Config Serialin = Buffered , Size = 10

'Configure pins used by this program
Config Portb = Output
Config Portd = Input
Config Pind.1 = Output
Config Pind.7 = Output
Config Pind.4 = Output
Config Pind.3 = Output
Config Pind.2 = Output
Config Pinb.2 = Output
Config Pinb.0 = Input
Portb.2 = 0
Portd.2 = 1

'Configure the LCD display
Config Lcdpin = Pin , Db4 = Pinc.2 , Db5 = Pinc.3 , Db6 = Pinc.4 , Db7 = Pinc.5 , E = Pinc.1 , Rs = Pinc.0

'***************************************************************************
'********************begin SERVO 1 AND SERVO2 CONTROLLER *******************
'***************************************************************************
'Configure the two servo's connected (to SONAR and to rudder)
'Servo(1) is the rudder
'Servo(2) is the direction the ultrasonic ranger
Config Servos = 2 , Servo1 = Portd.3 , Servo2 = Portd.4 , Reload = 100
'Enable the global interrupts (needed by the routine that controls the servos)
Enable Interrupts
'***************************************************************************
'********************end SERVO 1 AND SERVO 2 CONTROLLER*********************
'***************************************************************************

'Configure the Timer1 register, which is used to measure the time it takes for
'an echo to respond with the Polariod ranger
Config Timer1 = Timer , Prescale = 256
Stop Timer1




'Dimension the variables used by this program

Dim Temp As Integer
Dim Temp1 As Byte
Dim Temp2 As Byte
Dim Distance As Single
Dim Distr As Single
Dim Distl As Single
Dim Distc As Single

Dim Ann1 As Integer
Dim Ann2 As Integer
Dim Oleft As Bit
Dim Sturn As Eram Byte
Dim Oright As Bit
Dim Turn As Byte
Dim Panic As Bit
Dim Ruddertemp As Integer
Dim Rudder As Eram Integer
Dim Reset_now As Bit


'Varaibles and constants used by the SNAP protocol
Dim Message(9) As Byte
Const Myadd = 101
Const Sync = &B01010100


Main:

Ann1 = Rudder
Gosub Turner
Servo(1) = Ruddertemp
Cls
Temp1 = Sturn.0
Lcd Ruddertemp ; " " ; Temp1
Temp1 = Sturn.1
Lcd " " ; Temp1

'Temp = Getrc(pinb , 0)
'Distance = Temp / 2

'***************************************************************************
'********************begin TRANSDUCER CONTROLLER****************************
'***************************************************************************


Servo(2) = 14                           'center ranger
Waitms 300                              'wait for it to center
Gosub Sonar                             'get distance
Distc = Distance

Servo(2) = 12                           'point servo right
Waitms 350                              'wait for it to get there
Gosub Sonar                             'get distance
Distr = Distance

Servo(2) = 16                           'point servo left
Waitms 500                              'wait for it to get there
Gosub Sonar
Distl = Distance
Waitms 240

If Distr < 350 Then Oright = 1          'If there is an obstical close, don't
                                         'turn that way!!
Sturn.0 = Oright

If Distl < 350 Then Oleft = 1           'If there is an obstical close, don't
Sturn.1 = Oleft
                                        'turn that way!!


Cls
Temp = Distc
Lcd "C: " ; Temp
Temp = Distr
Lcd " R: " ; Temp
Temp = Distl
Lcd " L: " ; Temp

Waitms 200


'***************************************************************************
'**********************end TRANSDUCER CONTROLLER****************************
'***************************************************************************

Gosub Snappy                            'Send and recieve data to/from ANN board

Gosub Turner


Servo(1) = Ruddertemp

Lcd Ann1
Lowerline
Lcd Ann2 ; " " ; Ruddertemp ; " !"

Gosub Sendspeed

Portb.2 = 1

Goto Main

End                                     'end program


'***************************************************************************
'*********************begin DISTANCE CALCULATION****************************
'***************************************************************************
Sonar:

'Used for the Polaroid Spectra sonar
Stop Timer1
Timer1 = 0                              'Reset Timer1
Portd.2 = 0                             'Power up rangefinder board
Waitms 6                                'wait for it to stabalize
Portd.7 = 1                             'Fire pulse
Start Timer1                            'Start counting
Bitwait Pind.5 , Set                    'wait until echo is recieved
Stop Timer1                             'stop counting
Portd.7 = 0                             'power down board
Portd.2 = 1                             'power down board
Distance = Timer1 * 256                 'do some calculating
Distance = Distance / 242               'find distance in CM

'Distance = Distance - 100

'Used for the Polaroid Pronto Onestep sonar
'Timer1 = 0
'Portd.7 = 1                             'power up ranger board
'Bitwait Pind.6 , Set                    'wait for ranger to report "Pulse sent"
'Start Timer1                            'start timing
'Bitwait Pind.5 , Set                    'wait for ranger to report "Echo return"
'Stop Timer1                             'stop timing
'Portd.7 = 0                             'turn off ranger board
'Distance = Timer1 * 256                 'do some calculating
'Distance = Distance / 242               'find distance in CM

Return
'***************************************************************************
'*********************end DISTANCE CALCULATION******************************
'***************************************************************************


Turner:
'***************************************************************************
'***********************begin TURN DIRECTION LOGIC**************************
'***************************************************************************
If Oright = 1 Then Turn = 1             '1 = left
If Oleft = 1 Then Turn = 0              '0 = right
If Oright = 0 And Oleft = 0 And Ann1 < 300 Then
   Turn = Rnd(2)
End If

If Oright = 1 And Oleft = 1 Then Panic = 1
'***************************************************************************
'***********************end TURN DIRECTION LOGIC****************************
'***************************************************************************

'***************************************************************************
'***********************begin RUDDER POSITION CALCULATION*******************
'***************************************************************************

'turn the 0-1000 output of the ANN into a 6-12 value for the servo
'8 = centered (approximatly)
'10 = full turn
Ruddertemp = Ann1 / 200
Ruddertemp = Ruddertemp + 8

'If we want to turn the other way, reverse the above, to make a 8-4 value
'8 = centered
'4 = full turn
If Turn = 0 Then
  Ruddertemp = Ruddertemp * -1
  Ruddertemp = Ruddertemp + 18
End If

'***************************************************************************
'***********************end RUDDER POSITION CALCULATION*********************
'***************************************************************************
Return


Snappy:                                 'set of routines that use the SNAP
'"Scalable Node Address Protocol" Which Was Developed By High Tech Horizon
'The Main Snap Website Is www.hth.com/Snap

Sendann:                                'Send distance to ANN using SNAP
Temp = Distc

Message(1) = &B01010001                 'set up HDB2
Message(2) = &B00110010                 'set up HDB1
Message(3) = 100                        'set up Destination address
Message(4) = Myadd                      'set up Source address
Message(5) = High(temp)                 'Data Byte 2 - one byte of "distance"
Message(6) = Low(temp)                  'Data Byte 1 - second byte of "distance"
Temp1 = Crc8(message(1) , 6)            'Generate CRC8 byte
Message(7) = Temp1                      'Set up CRC8 byte

Printbin Sync ; Message(1) ; Message(2) ; Message(3) ; Message(4) ; Message(5) ; Message(6) ; Message(7)

'When the data is sent, an ACKnoledge should be send back indicating the packet
'was recieved okay, so now we have to wait for an acknoledge response

Annack:

Inputbin Temp1                          'Input byte
If Temp1 <> Sync Then Goto Annack       'is it sync byte?
Inputbin Temp1
Message(1) = Temp1
Inputbin Temp1
Message(2) = Temp1
Inputbin Temp1
Message(3) = Temp1
Inputbin Temp1
Message(4) = Temp1
Inputbin Temp1
Message(5) = Temp1
Inputbin Temp1
Message(6) = Temp1
Inputbin Temp1
Message(7) = Temp1

Temp1 = Crc8(message(1) , 6)            'do a CRC8 to check recieved data
If Temp1 <> Message(7) Then Goto Sendann

If Message(3) <> Myadd Then Goto Sendann       'check if message is for me
If Message(2) <> &B00110010 Then Goto Sendann       'check if format is correct

Temp1 = &B00000011
Temp2 = Temp1 And Message(1)
If Temp2 <> 2 Then Goto Sendann         'response FAILURE, resend data


'otherwise data should have been recieved okay, no need to resend it
'Now its time to recive the data from the ANN, which consits of two integers
Annget:
Inputbin Temp1
If Temp1 <> Sync Then Goto Annget
Inputbin Temp1
Message(1) = Temp1
Inputbin Temp1
Message(2) = Temp1
Inputbin Temp1
Message(3) = Temp1
Inputbin Temp1
Message(4) = Temp1
Inputbin Temp1
Message(5) = Temp1
Inputbin Temp1
Message(6) = Temp1
Inputbin Temp1
Message(7) = Temp1
Inputbin Temp1
Message(8) = Temp1
Inputbin Temp1
Message(9) = Temp1


Temp1 = Crc8(message(1) , 8)            'CRC8 check the data
If Temp1 <> Message(9) Then Goto Annnack

If Message(3) <> Myadd Then Goto Annget 'check data is for me
If Message(2) <> &B00110100 Then Goto Annget       'check format of message

Ann1 = Makeint(message(6) , Message(5)) 'make an integer out of the two bytes
Ann2 = Makeint(message(8) , Message(7)) 'make an integer out of the two bytes
Rudder = Ann1

'Send the acknoledge response

Message(1).1 = 1
Message(1).0 = 0
Gosub Respsend


Return

Annnack:
Message(1).1 = 1
Message(1).0 = 1
Gosub Respsend

Goto Annget


Respsend:

Message(3) = Message(4)
Message(4) = Myadd
Temp1 = Crc8(message(1) , 8)
Message(9) = Temp1
Printbin Sync ; Message(1) ; Message(2) ; Message(3) ; Message(4) ; Message(5) ; Message(6) ; Message(7) ; Message(8) ; Message(9)

Return

Sendspeed:

Message(1) = &B01010000
Message(2) = &B00110010
Message(3) = 102
Message(4) = Myadd
Temp1 = Ann2 / 100
'COMMENT BELOW LINE TO USE ANN
Temp1 = 20
Message(5) = Temp1
If Panic = 1 Then Message(6) = &B11110100 Else Message(6) = &B00100000
Temp1 = Crc8(message(1) , 6)
Message(7) = Temp1

Printbin Sync ; Message(1) ; Message(2) ; Message(3) ; Message(4) ; Message(5) ; Message(6) ; Message(7)

Return